function noPromise(){
    let resolve, reject;

    let np = new Promise((ok, fail) => {resolve = ok; reject = fail});

    np.resolve = resolve;
    np.reject  = reject;

    return np
}

function asynchronize({s, chunkEventName, endEventName}){
    return async function* wrapper(){
        const chunks        = {};
        const promises      = {};

        const clear = i => (delete chunks[i], delete promises[i])


        let   chunkCount    = 0;
        let   promiseCount  = 0;
        let   end           = false;

        if (!('on' in s)){
            s.on = function(eventName, callback){
                this['on' + eventName] = callback;
            }
        }

        s.on(chunkEventName, data => {
            chunks[chunkCount] = data

            if (chunkCount in promises){
                promises[chunkCount].resolve(chunks[chunkCount])
                clear(chunkCount)
            }
            chunkCount++
        })

        s.on(endEventName, () => {
            end = true;

            for (let i in promises){
                promises[i].reject(new Error('End Of S'))
            }
        })

        while (!end){
            let p = noPromise()

            if (promiseCount in chunks){
                p.resolve(chunks[promiseCount])
                clear(promiseCount)
            }

            promises[promiseCount] = p;
            promiseCount++

            yield p;
        }
    }
}

function msg2dom({nick, message, timestamp}){
    const div = document.createElement('div')
    div.innerHTML = `<b>${nick}<b>:${message}`
    return div;
}

let RPC = {
    addMessage(data){
        chat.appendChild(msg2dom(data))
    },

    getMessages({offset=0}){
        return messages.slice(offset)
    },

    getUserCount(){
        return userCount;
    }
}


const socket = new WebSocket("ws://localhost:8999/");
const aGena  = asynchronize({s: socket, chunkEventName: 'message', endEventName: 'close'});

(async () => {
    for await (let msg of aGena()){
        let data = JSON.parse(msg.data)
        if (data.func in RPC){
            RPC[data.func](data)
        }
    }
})()


send.onclick = function(){
    socket.send(JSON.stringify({func: 'addMessage', nick: nick.value, message: msg.value}))
}
